//
//  WritePDF.c
//  PDFCrashTest
//
//  Created by Mark Tucker on 2014-11-27.
//  Copyright (c) 2014 Cairo. All rights reserved.
//

#include "WritePDF.h"

#include <fstream>
#include <iostream>
#include <vector>

#include "cairo/cairo.h"
#include "cairo/cairo-pdf.h"

static cairo_status_t bufwrite(void* bp, unsigned char const* data, unsigned int len);

void WritePDF(const char* filePath)
{
    double ww = 595.276;
    double hh = 841.890;

    std::vector<uint8_t> buffer;
    buffer.reserve(128*1024*1024);
    cairo_surface_t* surface = cairo_pdf_surface_create_for_stream((cairo_write_func_t) bufwrite, (void*) &buffer, ww, hh);
    cairo_t* g = cairo_create(surface);

    try
    {
        std::cout << "Rendering..." << std::endl;
        cairo_set_line_width(g, 2);
        cairo_set_source_rgb(g, 0, 0, 0);
        cairo_move_to(g, 50, 50);
        cairo_line_to(g, 545.276, 250);
        cairo_stroke(g);

        cairo_set_source_rgb(g, 0.2, 0.2, 0.2);
        cairo_set_font_size(g, 14);
        cairo_move_to(g, 50, 300);
        cairo_show_text(g, "test string");
        cairo_fill(g);

        cairo_set_source_rgba(g, 0, 0, 0, 0.4);
        cairo_rectangle(g, 50, 450, 495.276, 200);
        cairo_fill(g);

        cairo_show_page(g);
    }
    catch (std::exception& e)
    {
        std::cout << "Caught exception in WritePDF: " << e.what() << std::endl;
    }
    catch (...)
    {
        std::cout << "Unknown exception caught in WritePDF." << std::endl;
    }

    cairo_destroy(g);
    cairo_surface_destroy(surface);

    std::cout << "Writing " << filePath << std::endl;
    std::ofstream f(filePath);
    f.write((const char*) &buffer[0], buffer.size());

    std::cout << "Done!" << std::endl;
}

static cairo_status_t bufwrite(void* bp, unsigned char const* data, unsigned int len)
{
    std::vector<uint8_t>& buffer = *((std::vector<uint8_t>*) bp);
    std::copy(data, data + len, std::back_inserter(buffer));
    return CAIRO_STATUS_SUCCESS;
}
